@@ -824,13 +824,28 @@ describe('MagicString', () => {
 			assert.equal(s.toString(), 'a&^...!?defghijkl');
 		});
 
-		it('disallows overwriting across moved content', () => {
+		it('disallows overwriting partially overlapping moved content', () => {
 			const s = new MagicString('abcdefghijkl');
 
 			s.move(6, 9, 3);
 			assert.throws(() => s.overwrite(5, 7, 'XX'), /Cannot overwrite across a split point/);
 		});
 
+		it('disallows overwriting fully surrounding content moved away', () => {
+			const s = new MagicString('abcdefghijkl');
+
+			s.move(6, 9, 3);
+			assert.throws(() => s.overwrite(4, 11, 'XX'), /Cannot overwrite across a split point/);
+		});
+
+		it('disallows overwriting fully surrounding content moved away even if there is another split', () => {
+			const s = new MagicString('abcdefghijkl');
+
+			s.move(6, 9, 3);
+			s.appendLeft(5, 'foo');
+			assert.throws(() => s.overwrite(4, 11, 'XX'), /Cannot overwrite across a split point/);
+		});
+
 		it('allows later insertions at the end', () => {
 			const s = new MagicString('abcdefg');
 
@@ -1262,4 +1277,74 @@ describe('MagicString', () => {
 			assert.equal(s.lastLine(), '//lastline');
 		});
 	});
+
+	describe('hasChanged', () => {
+		it('should works', () => {
+			const s = new MagicString(' abcde   fghijkl ');
+
+			assert.ok(!s.hasChanged());
+		
+			assert.ok(s.clone().prepend('  ').hasChanged());
+			assert.ok(s.clone().overwrite(1, 2, 'b').hasChanged());
+			assert.ok(s.clone().remove(1, 6).hasChanged());
+
+			s.trim();
+
+			assert.ok(s.hasChanged());
+
+			const clone = s.clone();
+
+			assert.ok(clone.hasChanged());
+		});
+	});
+  
+	describe('replace', () => {
+		it('works with string replace', () => {
+			const code = '1 2 1 2';
+			const s = new MagicString(code);
+
+			s.replace('2', '3');
+
+			assert.strictEqual(s.toString(), '1 3 1 2');
+		});
+
+		it('works with global regex replace', () => {
+			const s = new MagicString('1 2 3 4 a b c');
+
+			s.replace(/(\d)/g, 'xx$1$10');
+
+			assert.strictEqual(s.toString(), 'xx1$10 xx2$10 xx3$10 xx4$10 a b c');
+		});
+
+		it('works with global regex replace $$', () => {
+			const s = new MagicString('1 2 3 4 a b c');
+
+			s.replace(/(\d)/g, '$$');
+
+			assert.strictEqual(s.toString(),'$ $ $ $ a b c');
+		});
+
+		it('works with global regex replace function', () => {
+			const code = 'hey this is magic';
+			const s = new MagicString(code);
+
+			s.replace(/(\w)(\w+)/g, (_, $1, $2) => `${$1.toUpperCase()}${$2}`);
+
+			assert.strictEqual(s.toString(),'Hey This Is Magic');
+		});
+
+		it('replace function offset', () => {
+			// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/replace#specifying_a_function_as_a_parameter
+			function replacer(match, p1, p2, p3, offset, string, groups) {
+				// p1 is nondigits, p2 digits, and p3 non-alphanumerics
+				return [match, p1, p2, p3, offset, string, groups].join(' - ');
+			}
+			const code = 'abc12345#$*%';
+			const regex = /([^\d]*)(\d*)([^\w]*)/;
+			assert.strictEqual(
+				code.replace(regex, replacer),
+				new MagicString(code).replace(regex, replacer).toString()
+			);
+		});
+	});
 });
